#!/usr/bin/perl
use warnings;
use strict;

my $curr_block;
my @blocks;

while (my $l = <DATA>) {
	next unless ($l =~ /<dt>[^<]*<a/);
	# Some entries will be on two lines
	while (! ($l =~ m{</a></dt>}) ) { $l .= <DATA>; }
	$l =~ s/[\r\n]\s+/ /g;

	# Have chapter numbers ($leader) outside the link
	my ($indent, $leader, $url, $title) = $l =~ /^(\s+)<dt>([^<]*)<a \s+ href \s* = \s* "([^"]+)">(.+?)<\/a><\/dt>/x;
	die "Bad line: $l\ni/u/t = $indent/$url/$title" unless ($indent && $url && $title);
	$indent = length($indent);
	$leader = '' unless ($leader);
	$title  = $leader.$title;
	$title =~ s/<[^>]+>//g;

	#print "$indent\t$url\t$title\n";
	if ($indent eq 4) {
		# new block
		if ($curr_block) { push @blocks, $curr_block; }
		$curr_block = { title_url => $url, title_text => $title, items => [] };
	}
	elsif ($indent eq 8) {
		my %item = ( url => $url, text => $title );
		push @{$curr_block->{items}}, \%item;
	}
	else {
		die "Unexpected indent: $indent : $url : $title";
	}
}
if ($curr_block) { push @blocks, $curr_block; }

# OK, let's re-render them...
my $section_links = '';
my $chapter_links = '';
for my $b (@blocks) {
	my $section_id = 'menu-'.$b->{title_url};
	$section_id    =~ s/.html$//;

	$section_links .= '<a href="#'.$section_id.'">'.$b->{title_text}.'</a>';

	$chapter_links .= '<div id="'.$section_id.'" class="menusection">';
	$chapter_links .= '<h2><a href="'.$b->{title_url}.'">'.$b->{title_text}.'</a></h2>';
	for my $i (@{$b->{items}}) {
		$chapter_links .= '<a href="' . $i->{url} . '">' . $i->{text} . '</a>';
	}
	$chapter_links .= '</div>';
}
print '<div class="contents">';
print '<div id="menu-section-links">'.$section_links.'</div>';
print '<div id="menu-chapter-links">'.$chapter_links.'</div>';
print '</div>';

exit;

__DATA__
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01//EN">

<html>
<head>
  <meta name="generator" content=
  "HTML Tidy for Linux/x86 (vers 1 September 2005), see www.w3.org">

  <title></title>
</head>

<body>
  <dl>
    <dt><a href="preface.html">Preface</a></dt>

    <dd>
      <dl>
        <dt><a href="intro-whatis.html">What is <span class=
        "PRODUCTNAME">PostgreSQL</span>?</a></dt>

        <dt><a href="history.html">A Brief History of <span class=
        "PRODUCTNAME">PostgreSQL</span></a></dt>

        <dt><a href="notation.html">Conventions</a></dt>

        <dt><a href="resources.html">Further Information</a></dt>

        <dt><a href="bug-reporting.html">Bug Reporting
        Guidelines</a></dt>
      </dl>
    </dd>

    <dt>I. <a href="tutorial.html">Tutorial</a></dt>

    <dd>
      <dl>
        <dt>1. <a href="tutorial-start.html">Getting
        Started</a></dt>

        <dt>2. <a href="tutorial-sql.html">The <acronym class=
        "ACRONYM">SQL</acronym> Language</a></dt>

        <dt>3. <a href="tutorial-advanced.html">Advanced
        Features</a></dt>
      </dl>
    </dd>

    <dt>II. <a href="sql.html">The SQL Language</a></dt>

    <dd>
      <dl>
        <dt>4. <a href="sql-syntax.html">SQL Syntax</a></dt>

        <dt>5. <a href="ddl.html">Data Definition</a></dt>

        <dt>6. <a href="dml.html">Data Manipulation</a></dt>

        <dt>7. <a href="queries.html">Queries</a></dt>

        <dt>8. <a href="datatype.html">Data Types</a></dt>

        <dt>9. <a href="functions.html">Functions and
        Operators</a></dt>

        <dt>10. <a href="typeconv.html">Type Conversion</a></dt>

        <dt>11. <a href="indexes.html">Indexes</a></dt>

        <dt>12. <a href="textsearch.html">Full Text Search</a></dt>

        <dt>13. <a href="mvcc.html">Concurrency Control</a></dt>

        <dt>14. <a href="performance-tips.html">Performance
        Tips</a></dt>
      </dl>
    </dd>

    <dt>III. <a href="admin.html">Server Administration</a></dt>

    <dd>
      <dl>
        <dt>15. <a href="installation.html">Installation
        Instructions</a></dt>

        <dt>16. <a href="install-win32.html">Installation on
        <span class="PRODUCTNAME">Windows</span></a></dt>

        <dt>17. <a href="runtime.html">Operating System
        Environment</a></dt>

        <dt>18. <a href="runtime-config.html">Server
        Configuration</a></dt>

        <dt>19. <a href="user-manag.html">Database Roles and
        Privileges</a></dt>

        <dt>20. <a href="managing-databases.html">Managing
        Databases</a></dt>

        <dt>21. <a href="client-authentication.html">Client
        Authentication</a></dt>

        <dt>22. <a href="charset.html">Localization</a></dt>

        <dt>23. <a href="maintenance.html">Routine Database
        Maintenance Tasks</a></dt>

        <dt>24. <a href="backup.html">Backup and Restore</a></dt>

        <dt>25. <a href="high-availability.html">High Availability,
        Load Balancing, and Replication</a></dt>

        <dt>26. <a href="monitoring.html">Monitoring Database
        Activity</a></dt>

        <dt>27. <a href="diskusage.html">Monitoring Disk
        Usage</a></dt>

        <dt>28. <a href="wal.html">Reliability and the Write-Ahead
        Log</a></dt>

        <dt>29. <a href="regress.html">Regression Tests</a></dt>
      </dl>
    </dd>

    <dt>IV. <a href="client-interfaces.html">Client
    Interfaces</a></dt>

    <dd>
      <dl>
        <dt>30. <a href="libpq.html"><span class=
        "APPLICATION">libpq</span> - C Library</a></dt>

        <dt>31. <a href="largeobjects.html">Large Objects</a></dt>

        <dt>32. <a href="ecpg.html"><span class=
        "APPLICATION">ECPG</span> - Embedded <acronym class=
        "ACRONYM">SQL</acronym> in C</a></dt>

        <dt>33. <a href="information-schema.html">The Information
        Schema</a></dt>
      </dl>
    </dd>

    <dt>V. <a href="server-programming.html">Server
    Programming</a></dt>

    <dd>
      <dl>
        <dt>34. <a href="extend.html">Extending <acronym class=
        "ACRONYM">SQL</acronym></a></dt>

        <dt>35. <a href="triggers.html">Triggers</a></dt>

        <dt>36. <a href="rules.html">The Rule System</a></dt>

        <dt>37. <a href="xplang.html">Procedural Languages</a></dt>

        <dt>38. <a href="plpgsql.html"><span class=
        "APPLICATION">PL/pgSQL</span> - <acronym class=
        "ACRONYM">SQL</acronym> Procedural Language</a></dt>

        <dt>39. <a href="pltcl.html">PL/Tcl - Tcl Procedural
        Language</a></dt>

        <dt>40. <a href="plperl.html">PL/Perl - Perl Procedural
        Language</a></dt>

        <dt>41. <a href="plpython.html">PL/Python - Python
        Procedural Language</a></dt>

        <dt>42. <a href="spi.html">Server Programming
        Interface</a></dt>
      </dl>
    </dd>

    <dt>VI. <a href="reference.html">Reference</a></dt>

    <dd>
      <dl>
        <dt>I. <a href="sql-commands.html">SQL Commands</a></dt>

        <dt>II. <a href="reference-client.html">PostgreSQL Client
        Applications</a></dt>

        <dt>III. <a href="reference-server.html">PostgreSQL Server
        Applications</a></dt>
      </dl>
    </dd>

    <dt>VII. <a href="internals.html">Internals</a></dt>

    <dd>
      <dl>
        <dt>43. <a href="overview.html">Overview of PostgreSQL
        Internals</a></dt>

        <dt>44. <a href="catalogs.html">System Catalogs</a></dt>

        <dt>45. <a href="protocol.html">Frontend/Backend
        Protocol</a></dt>

        <dt>46. <a href="source.html">PostgreSQL Coding
        Conventions</a></dt>

        <dt>47. <a href="nls.html">Native Language Support</a></dt>

        <dt>48. <a href="plhandler.html">Writing A Procedural
        Language Handler</a></dt>

        <dt>49. <a href="geqo.html">Genetic Query
        Optimizer</a></dt>

        <dt>50. <a href="indexam.html">Index Access Method
        Interface Definition</a></dt>

        <dt>51. <a href="gist.html">GiST Indexes</a></dt>

        <dt>52. <a href="gin.html">GIN Indexes</a></dt>

        <dt>53. <a href="storage.html">Database Physical
        Storage</a></dt>

        <dt>54. <a href="bki.html"><acronym class=
        "ACRONYM">BKI</acronym> Backend Interface</a></dt>

        <dt>55. <a href="planner-stats-details.html">How the
        Planner Uses Statistics</a></dt>
      </dl>
    </dd>

    <dt>VIII. <a href="appendixes.html">Appendixes</a></dt>

    <dd>
      <dl>
        <dt>A. <a href="errcodes-appendix.html"><span class=
        "PRODUCTNAME">PostgreSQL</span> Error Codes</a></dt>

        <dt>B. <a href="datetime-appendix.html">Date/Time
        Support</a></dt>

        <dt>C. <a href="sql-keywords-appendix.html"><acronym class=
        "ACRONYM">SQL</acronym> Key Words</a></dt>

        <dt>D. <a href="features.html">SQL Conformance</a></dt>

        <dt>E. <a href="release.html">Release Notes</a></dt>

        <dt>F. <a href="contrib.html">Additional Supplied
        Modules</a></dt>

        <dt>G. <a href="external-projects.html">External
        Projects</a></dt>

        <dt>H. <a href="cvs.html">The <span class=
        "PRODUCTNAME">CVS</span> Repository</a></dt>

        <dt>I. <a href="docguide.html">Documentation</a></dt>

        <dt>J. <a href="acronyms.html">Acronyms</a></dt>
      </dl>
    </dd>

    <dt><a href="biblio.html">Bibliography</a></dt>

    <dt><a href="bookindex.html">Index</a></dt>
  </dl>
</body>
</html>
