#!/usr/bin/perl 

use strict;
use warnings;

use IO::File;

if (<*.BAK>) {
	die "Cowardly refusing to run while .BAK files still around";
}

# Parse the main contents page for link-targets and titles
my %link_titles = build_page_titles();

# For each target file, scan for <a href="some target file"> and set its title attribute
for my $k (keys %link_titles) {
	print "$k : $link_titles{$k}\n";
	title_links_in_file($k, \%link_titles);
}

exit;

sub build_page_titles {
	my %link_titles;

	for my $filename (<*.html>) {
		my $fh = new IO::File($filename, 'r') or die "Unable to open $filename for reading: $!";
		local( $/ );
		my $file = <$fh>;
		$fh->close or die "Unable to close index.html: $!";

		my ($page_title) = $file =~ /<H1 [^>]+> (.+?) <\/H1/xms;

		# Not every file has a title (LEGALNOTICE.html)
		next unless ($page_title);
		# Strip any <span>s and newlines
		$page_title =~ s/<[^>]+>//g;
		$page_title =~ tr/\r\n//;
		$link_titles{$filename} = $page_title;
	}

	return %link_titles;
}

sub title_links_in_file {
	my ($filename, $link_titles) = @_;

	my $fh = new IO::File($filename, 'r') or die "Unable to open $filename for reading: $!";
	local( $/ );
	my $file = <$fh>;
	$fh->close or die "Unable to close $filename: $!";

	rename($filename, "$filename.BAK") or die "Unable to backup $filename: $!";

	my ($header, $body, $footer) = split(/<HR/, $file);
	$body = '<HR'.$body;
	$footer = '<HR'.$footer;

	for my $url (keys %$link_titles) {
		my $t = $link_titles->{$url};
		$header =~ s/<A \s+ HREF="$url"/$& TITLE="$t"/xmsg;
	}

	$fh = new IO::File($filename, 'w') or die "Unable to open $filename for writing: $!";
	print $fh $header, $body, $footer;
	$fh->close or die "Unable to close $filename: $!";
}
